@@ -2006,7 +2006,10 @@ void structure::change_residue(residue &res, const std::string &newCompound,
 			continue;
 
 		if (a2.empty() or a2 == ".")
+		{
+			i->set_property("label_comp_id", newCompound);
 			remove_atom(*i);
+		}
 		else if (a1 != a2)
 		{
 			auto ra = r.front();
@@ -2833,15 +2836,4 @@ void structure::validate_atoms() const
 	assert(atoms.empty());
 }
 
-// --------------------------------------------------------------------
-
-void reconstruct_pdbx(datablock &db)
-{
-	if (db.get("atom_site") == nullptr)
-		throw std::runtime_error("Cannot reconstruct PDBx file, atom data missing");
-	
-	assert(false);
-	throw std::runtime_error("not implemented yet");
-}
-
 } // namespace pdbx
