@@ -1,9 +1,11 @@
 #include "cif++/utilities.hpp"
 
+#include <catch2/catch.hpp>
+
 #include <random>
 #include <thread>
 
-void test_one()
+TEST_CASE("test_one")
 {
 	std::random_device rd;
     std::mt19937 gen(rd());
@@ -20,7 +22,7 @@ void test_one()
 	}
 }
 
-void test_two()
+TEST_CASE("test_two")
 {
 	cif::progress_bar pb(10, "test");
 
@@ -29,7 +31,7 @@ void test_two()
 		pb.consumed(1);
 }
 
-void test_three()
+TEST_CASE("test_three")
 {
 	using namespace std::literals;
 
@@ -38,12 +40,3 @@ void test_three()
 
 	std::this_thread::sleep_for(100ms);
 }
-
-int main()
-{
-	test_one();
-	test_two();
-	test_three();
-
-	return 0;
-}
\ No newline at end of file
