@@ -24,8 +24,9 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#define BOOST_TEST_ALTERNATIVE_INIT_API
-#include <boost/test/included/unit_test.hpp>
+#include "test-main.hpp"
+
+#include <catch2/catch.hpp>
 
 #include <stdexcept>
 
@@ -33,11 +34,6 @@
 
 #include <Eigen/Eigenvalues>
 
-namespace tt = boost::test_tools;
-namespace utf = boost::unit_test;
-
-std::filesystem::path gTestDir = std::filesystem::current_path(); // filled in first test
-
 // --------------------------------------------------------------------
 
 cif::file operator""_cf(const char *text, size_t length)
@@ -56,28 +52,9 @@ cif::file operator""_cf(const char *text, size_t length)
 
 // --------------------------------------------------------------------
 
-bool init_unit_test()
-{
-	cif::VERBOSE = 1;
-
-	// not a test, just initialize test dir
-	if (boost::unit_test::framework::master_test_suite().argc == 2)
-		gTestDir = boost::unit_test::framework::master_test_suite().argv[1];
-
-	// do this now, avoids the need for installing
-	cif::add_file_resource("mmcif_pdbx.dic", gTestDir / ".." / "rsrc" / "mmcif_pdbx.dic");
-
-	// initialize CCD location
-	cif::add_file_resource("components.cif", gTestDir / ".." / "data" / "ccd-subset.cif");
-
-	return true;
-}
-
-// --------------------------------------------------------------------
-
 // 3d tests
 
-BOOST_AUTO_TEST_CASE(t1)
+TEST_CASE("t1")
 {
 	// std::random_device rnd;
 	// std::mt19937 gen(rnd());
@@ -115,19 +92,19 @@ BOOST_AUTO_TEST_CASE(t1)
 
 	const auto &&[angle, axis] = cif::quaternion_to_angle_axis(q2);
 
-	BOOST_TEST(std::fmod(360 + angle, 360) == std::fmod(360 - angle0, 360), tt::tolerance(0.01));
+	REQUIRE_THAT(std::fmod(360 + angle, 360), Catch::Matchers::WithinRel(std::fmod(360 - angle0, 360), 0.01));
 
 	for (auto &p : p1)
 		p.rotate(q2);
 
 	auto rmsd = cif::RMSd(p1, p2);
 
-	BOOST_TEST(rmsd < 1e-5);
+	REQUIRE(rmsd < 1e-5);
 
 	// std::cout << "rmsd: " << RMSd(p1, p2) << '\n';
 }
 
-BOOST_AUTO_TEST_CASE(t2)
+TEST_CASE("t2")
 {
 	cif::point p[] = {
 		{ 1, 1, 0 },
@@ -141,10 +118,10 @@ BOOST_AUTO_TEST_CASE(t2)
 
 	auto &&[angle, axis] = cif::quaternion_to_angle_axis(q);
 
-	BOOST_TEST(angle == 45, tt::tolerance(0.01));
+	REQUIRE_THAT(angle, Catch::Matchers::WithinRel(45.f, 0.01f));
 }
 
-BOOST_AUTO_TEST_CASE(t3)
+TEST_CASE("t3")
 {
 	cif::point p[] = {
 		{ 1, 1, 0 },
@@ -165,10 +142,10 @@ BOOST_AUTO_TEST_CASE(t3)
 
 	double a = cif::angle(v, p[0], p[1]);
 
-	BOOST_TEST(a == 45, tt::tolerance(0.01));
+	REQUIRE_THAT(a, Catch::Matchers::WithinRel(45.f, 0.01f));
 }
 
-BOOST_AUTO_TEST_CASE(dh_q_0)
+TEST_CASE("dh_q_0")
 {
 	cif::point axis(1, 0, 0);
 
@@ -182,33 +159,33 @@ BOOST_AUTO_TEST_CASE(dh_q_0)
 	};
 
 	auto a = cif::dihedral_angle(t[0], t[1], t[2], p);
-	BOOST_TEST(a == 0, tt::tolerance(0.01f));
+	REQUIRE_THAT(a, Catch::Matchers::WithinRel(0, 0.01f));
 
 	auto q = cif::construct_from_angle_axis(90, axis);
 
 	p.rotate(q);
 
-	BOOST_TEST(p.m_x == 1, tt::tolerance(0.01f));
-	BOOST_TEST(p.m_y == 0, tt::tolerance(0.01f));
-	BOOST_TEST(p.m_z == 1, tt::tolerance(0.01f));
+	REQUIRE_THAT(p.m_x, Catch::Matchers::WithinRel(1, 0.01f));
+	REQUIRE_THAT(p.m_y, Catch::Matchers::WithinRel(0, 0.01f));
+	REQUIRE_THAT(p.m_z, Catch::Matchers::WithinRel(1, 0.01f));
 
 	a = cif::dihedral_angle(t[0], t[1], t[2], p);
-	BOOST_TEST(a == 90, tt::tolerance(0.01f));
+	REQUIRE_THAT(a, Catch::Matchers::WithinRel(90, 0.01f));
 
 	q = cif::construct_from_angle_axis(-90, axis);
 
 	p.rotate(q);
 
-	BOOST_TEST(p.m_x == 1, tt::tolerance(0.01f));
-	BOOST_TEST(p.m_y == 1, tt::tolerance(0.01f));
-	BOOST_TEST(p.m_z == 0, tt::tolerance(0.01f));
+	REQUIRE_THAT(p.m_x, Catch::Matchers::WithinRel(1, 0.01f));
+	REQUIRE_THAT(p.m_y, Catch::Matchers::WithinRel(1, 0.01f));
+	REQUIRE_THAT(p.m_z, Catch::Matchers::WithinRel(0, 0.01f));
 
 	a = cif::dihedral_angle(t[0], t[1], t[2], p);
-	BOOST_TEST(a == 0, tt::tolerance(0.01f));
+	REQUIRE_THAT(a, Catch::Matchers::WithinRel(0, 0.01f));
 
 }
 
-BOOST_AUTO_TEST_CASE(dh_q_1)
+TEST_CASE("dh_q_1")
 {
 	struct
 	{
@@ -247,13 +224,13 @@ BOOST_AUTO_TEST_CASE(dh_q_1)
 		pts[3].rotate(q, pts[2]);
 
 		auto dh = cif::dihedral_angle(pts[0], pts[1], pts[2], pts[3]);
-		BOOST_TEST(dh == angle, tt::tolerance(0.1f));
+		REQUIRE_THAT(dh, Catch::Matchers::WithinRel(angle, 0.1f));
 	}
 }
 
 // --------------------------------------------------------------------
 
-BOOST_AUTO_TEST_CASE(m2q_0, *utf::tolerance(0.001f))
+TEST_CASE("m2q_0, *utf::tolerance(0.001f)")
 {
 	for (size_t i = 0; i < cif::kSymopNrTableSize; ++i)
 	{
@@ -310,13 +287,13 @@ BOOST_AUTO_TEST_CASE(m2q_0, *utf::tolerance(0.001f))
 
 		cif::point p3 = rot * p1;
 
-		BOOST_TEST(p2.m_x == p3.m_x);
-		BOOST_TEST(p2.m_y == p3.m_y);
-		BOOST_TEST(p2.m_z == p3.m_z);
+		REQUIRE_THAT(p2.m_x, Catch::Matchers::WithinRel(p3.m_x, 0.01f));
+		REQUIRE_THAT(p2.m_y, Catch::Matchers::WithinRel(p3.m_y, 0.01f));
+		REQUIRE_THAT(p2.m_z, Catch::Matchers::WithinRel(p3.m_z, 0.01f));
 	}
 }
 
-// BOOST_AUTO_TEST_CASE(m2q_1, *utf::tolerance(0.001f))
+// "TEST_CASE(m2q_1, *utf::tolerance(0.001f)")
 // {
 // 	for (size_t i = 0; i < cif::kSymopNrTableSize; ++i)
 // 	{
@@ -369,15 +346,15 @@ BOOST_AUTO_TEST_CASE(m2q_0, *utf::tolerance(0.001f))
 
 // 		cif::point p3 = rot * p1;
 
-// 		BOOST_TEST(p2.m_x == p3.m_x);
-// 		BOOST_TEST(p2.m_y == p3.m_y);
-// 		BOOST_TEST(p2.m_z == p3.m_z);
+// 		REQUIRE(p2.m_x == p3.m_x);
+// 		REQUIRE(p2.m_y == p3.m_y);
+// 		REQUIRE(p2.m_z == p3.m_z);
 // 	}
 // }
 
 // --------------------------------------------------------------------
 
-BOOST_AUTO_TEST_CASE(symm_1)
+TEST_CASE("symm_1")
 {
 	cif::cell c(10, 10, 10);
 
@@ -385,37 +362,37 @@ BOOST_AUTO_TEST_CASE(symm_1)
 
 	cif::point f = fractional(p, c);
 
-	BOOST_TEST(f.m_x == 0.1f, tt::tolerance(0.01));
-	BOOST_TEST(f.m_y == 0.1f, tt::tolerance(0.01));
-	BOOST_TEST(f.m_z == 0.1f, tt::tolerance(0.01));
+	REQUIRE_THAT(f.m_x, Catch::Matchers::WithinRel(0.1f, 0.01f));
+	REQUIRE_THAT(f.m_y, Catch::Matchers::WithinRel(0.1f, 0.01f));
+	REQUIRE_THAT(f.m_z, Catch::Matchers::WithinRel(0.1f, 0.01f));
 
 	cif::point o = orthogonal(f, c);
 
-	BOOST_TEST(o.m_x == 1.f, tt::tolerance(0.01));
-	BOOST_TEST(o.m_y == 1.f, tt::tolerance(0.01));
-	BOOST_TEST(o.m_z == 1.f, tt::tolerance(0.01));
+	REQUIRE_THAT(o.m_x, Catch::Matchers::WithinRel(1.f, 0.01f));
+	REQUIRE_THAT(o.m_y, Catch::Matchers::WithinRel(1.f, 0.01f));
+	REQUIRE_THAT(o.m_z, Catch::Matchers::WithinRel(1.f, 0.01f));
 }
 
-BOOST_AUTO_TEST_CASE(symm_2)
+TEST_CASE("symm_2")
 {
 	using namespace cif::literals;
 
 	auto symop = "1_555"_symop;
 
-	BOOST_TEST(symop.is_identity() == true);
+	REQUIRE(symop.is_identity() == true);
 }
 
-BOOST_AUTO_TEST_CASE(symm_3)
+TEST_CASE("symm_3")
 {
 	using namespace cif::literals;
 
 	cif::spacegroup sg(18);
 
-	BOOST_TEST(sg.size() == 4);
-	BOOST_TEST(sg.get_name() == "P 21 21 2");
+	REQUIRE(sg.size() == 4UL);
+	REQUIRE(sg.get_name() == "P 21 21 2");
 }
 
-BOOST_AUTO_TEST_CASE(symm_4, *utf::tolerance(0.1f))
+TEST_CASE("symm_4, *utf::tolerance(0.1f)")
 {
 	using namespace cif::literals;
 
@@ -427,21 +404,21 @@ BOOST_AUTO_TEST_CASE(symm_4, *utf::tolerance(0.1f))
 	cif::point b{  -35.356,  33.693, -3.236 }; // CG2 THR D 400
 	cif::point sb(  -6.916,   79.34,   3.236); // 4_565 copy of b
 
-	BOOST_TEST(distance(a, sg(a, c, "1_455"_symop)) == static_cast<float>(c.get_a()));
-	BOOST_TEST(distance(a, sg(a, c, "1_545"_symop)) == static_cast<float>(c.get_b()));
-	BOOST_TEST(distance(a, sg(a, c, "1_554"_symop)) == static_cast<float>(c.get_c()));
+	REQUIRE_THAT(distance(a, sg(a, c, "1_455"_symop)), Catch::Matchers::WithinRel(static_cast<float>(c.get_a()), 0.01f));
+	REQUIRE_THAT(distance(a, sg(a, c, "1_545"_symop)), Catch::Matchers::WithinRel(static_cast<float>(c.get_b()), 0.01f));
+	REQUIRE_THAT(distance(a, sg(a, c, "1_554"_symop)), Catch::Matchers::WithinRel(static_cast<float>(c.get_c()), 0.01f));
 
 	auto sb2 = sg(b, c, "4_565"_symop);
-	BOOST_TEST(sb.m_x == sb2.m_x);
-	BOOST_TEST(sb.m_y == sb2.m_y);
-	BOOST_TEST(sb.m_z == sb2.m_z);
+	REQUIRE_THAT(sb.m_x, Catch::Matchers::WithinRel(sb2.m_x, 0.01f));
+	REQUIRE_THAT(sb.m_y, Catch::Matchers::WithinRel(sb2.m_y, 0.01f));
+	REQUIRE_THAT(sb.m_z, Catch::Matchers::WithinRel(sb2.m_z, 0.01f));
 
-	BOOST_TEST(distance(a, sb2) == 7.42f);	
+	REQUIRE_THAT(distance(a, sb2), Catch::Matchers::WithinRel(7.42f, 0.01f));	
 }
 
 // --------------------------------------------------------------------
 
-BOOST_AUTO_TEST_CASE(symm_4wvp_1, *utf::tolerance(0.1f))
+TEST_CASE("symm_4wvp_1, *utf::tolerance(0.1f)")
 {
 	using namespace cif::literals;
 
@@ -456,21 +433,21 @@ BOOST_AUTO_TEST_CASE(symm_4wvp_1, *utf::tolerance(0.1f))
 	auto a = s.get_residue("A", 10, "").get_atom_by_atom_id("O");
 
 	auto sp1 = c.symmetry_copy(a.get_location(), "2_565"_symop);
-	BOOST_TEST(sp1.m_x == p.m_x);
-	BOOST_TEST(sp1.m_y == p.m_y);
-	BOOST_TEST(sp1.m_z == p.m_z);
+	REQUIRE_THAT(sp1.m_x, Catch::Matchers::WithinAbs(p.m_x, 0.5f));
+	REQUIRE_THAT(sp1.m_y, Catch::Matchers::WithinAbs(p.m_y, 0.5f));
+	REQUIRE_THAT(sp1.m_z, Catch::Matchers::WithinAbs(p.m_z, 0.5f));
 
 	const auto &[d, sp2, so] = c.closest_symmetry_copy(p, a.get_location());
 
-	BOOST_TEST(d < 1);
+	REQUIRE(d < 1);
 
-	BOOST_TEST(sp2.m_x == p.m_x);
-	BOOST_TEST(sp2.m_y == p.m_y);
-	BOOST_TEST(sp2.m_z == p.m_z);
+	REQUIRE_THAT(sp2.m_x, Catch::Matchers::WithinAbs(p.m_x, 0.5f));
+	REQUIRE_THAT(sp2.m_y, Catch::Matchers::WithinAbs(p.m_y, 0.5f));
+	REQUIRE_THAT(sp2.m_z, Catch::Matchers::WithinAbs(p.m_z, 0.5f));
 
 }
 
-BOOST_AUTO_TEST_CASE(symm_2bi3_1, *utf::tolerance(0.1f))
+TEST_CASE("symm_2bi3_1, *utf::tolerance(0.1f)")
 {
 	cif::file f(gTestDir / "2bi3.cif.gz");
 
@@ -502,22 +479,22 @@ BOOST_AUTO_TEST_CASE(symm_2bi3_1, *utf::tolerance(0.1f))
 		auto sa1 = c.symmetry_copy(a1.get_location(), cif::sym_op(symm1));
 		auto sa2 = c.symmetry_copy(a2.get_location(), cif::sym_op(symm2));
 
-		BOOST_TEST(cif::distance(sa1, sa2) == dist);
+		REQUIRE_THAT(cif::distance(sa1, sa2), Catch::Matchers::WithinAbs(dist, 0.5f));
 
 		auto pa1 = a1.get_location();
 
 		const auto &[d, p, so] = c.closest_symmetry_copy(pa1, a2.get_location());
 
-		BOOST_TEST(p.m_x == sa2.m_x);
-		BOOST_TEST(p.m_y == sa2.m_y);
-		BOOST_TEST(p.m_z == sa2.m_z);
+		REQUIRE_THAT(p.m_x, Catch::Matchers::WithinAbs(sa2.m_x, 0.5f));
+		REQUIRE_THAT(p.m_y, Catch::Matchers::WithinAbs(sa2.m_y, 0.5f));
+		REQUIRE_THAT(p.m_z, Catch::Matchers::WithinAbs(sa2.m_z, 0.5f));
 
-		BOOST_TEST(d == dist);
-		BOOST_TEST(so.string() == symm2);
+		REQUIRE_THAT(d, Catch::Matchers::WithinAbs(dist, 0.5f));
+		REQUIRE(so.string() == symm2);
 	}
 }
 
-BOOST_AUTO_TEST_CASE(symm_2bi3_1a, *utf::tolerance(0.1f))
+TEST_CASE("symm_2bi3_1a, *utf::tolerance(0.1f)")
 {
 	using namespace cif::literals;
 
@@ -552,20 +529,20 @@ BOOST_AUTO_TEST_CASE(symm_2bi3_1a, *utf::tolerance(0.1f))
 		auto sa1 = c.symmetry_copy(p1, cif::sym_op(symm1));
 		auto sa2 = c.symmetry_copy(p2, cif::sym_op(symm2));
 
-		BOOST_TEST(cif::distance(sa1, sa2) == dist);
+		REQUIRE_THAT(cif::distance(sa1, sa2), Catch::Matchers::WithinAbs(dist, 0.5f));
 
 		const auto &[d, p, so] = c.closest_symmetry_copy(p1, p2);
 
-		BOOST_TEST(p.m_x == sa2.m_x);
-		BOOST_TEST(p.m_y == sa2.m_y);
-		BOOST_TEST(p.m_z == sa2.m_z);
+		REQUIRE_THAT(p.m_x, Catch::Matchers::WithinAbs(sa2.m_x, 0.5f));
+		REQUIRE_THAT(p.m_y, Catch::Matchers::WithinAbs(sa2.m_y, 0.5f));
+		REQUIRE_THAT(p.m_z, Catch::Matchers::WithinAbs(sa2.m_z, 0.5f));
 
-		BOOST_TEST(d == dist);
-		BOOST_TEST(so.string() == symm2);
+		REQUIRE_THAT(d, Catch::Matchers::WithinAbs(dist, 0.5f));
+		REQUIRE(so.string() == symm2);
 	}
 }
 
-BOOST_AUTO_TEST_CASE(symm_3bwh_1, *utf::tolerance(0.1f))
+TEST_CASE("symm_3bwh_1, *utf::tolerance(0.1f)")
 {
 	cif::file f(gTestDir / "3bwh.cif.gz");
 
@@ -583,12 +560,12 @@ BOOST_AUTO_TEST_CASE(symm_3bwh_1, *utf::tolerance(0.1f))
 			
 			const auto&[ d, p, so ] = c.closest_symmetry_copy(a1.get_location(), a2.get_location());
 
-			BOOST_TEST(d == distance(a1.get_location(), p));
+			REQUIRE_THAT(d, Catch::Matchers::WithinAbs(distance(a1.get_location(), p), 0.5f));
 		}
 	}
 }
 
-BOOST_AUTO_TEST_CASE(volume_3bwh_1, *utf::tolerance(0.1f))
+TEST_CASE("volume_3bwh_1, *utf::tolerance(0.1f)")
 {
 	cif::file f(gTestDir / "1juh.cif.gz");
 
@@ -596,6 +573,6 @@ BOOST_AUTO_TEST_CASE(volume_3bwh_1, *utf::tolerance(0.1f))
 
 	cif::crystal c(db);
 
-	BOOST_CHECK_EQUAL(c.get_cell().get_volume(), 741009.625f);
+	REQUIRE_THAT(c.get_cell().get_volume(), Catch::Matchers::WithinRel(741009.625f, 0.01f));
 }
 
